@@ -828,7 +828,7 @@ class GptManager:
         """
         if self.event == "-COLLATE_RESPONSES_BOOL-":
             if self.window_mgr.get_values()["-COLLATE_BOOL_RESPONSES-"]:
-                files_list = self.window['-FILES_LIST_RESPONSES-'].Values
+                files_list = self.browser_mgr.files_list_mgr.get_file_path_list(key='responses')
                 response_key = self.window_mgr.get_values()['-KEY_SELECTION_RESPONSES-'] or 'api_response'
                 if len(files_list)>0:
                     collator = collate_responses(files_list,response_key)
@@ -970,7 +970,7 @@ class GptManager:
         if self.script_event_js['found'] == '-FILES_LIST-':
             selected_files = values[self.event]
             if selected_files:
-                file_path = selected_files[0]
+                file_path = self.browser_mgr.files_list_mgr.get_file_path(key='responses',display_number=selected_files[0])
                 event_triggered = True
         elif self.script_event_js['found'] == '-BROWSER_LIST-':
             directory = values[self.script_event_js['-DIR-']]
@@ -1027,17 +1027,11 @@ class GptManager:
         if self.latest_output:
             for data in self.latest_output:
                 file_path = get_any_value(data,'file_path')
-                if file_path in self.new_response_path_list:
-                    self.new_response_path_list.remove(file_path)
-                self.new_response_path_list.append(file_path)
-            files_list = self.window['-FILES_LIST_RESPONSES-'].Values
-            if file_path in self.new_response_path_list:
-                if file_path in files_list:
-                    files_list.remove(file_path)
-            files_list=self.new_response_path_list+files_list
-            self.window_mgr.update_value('-FILES_LIST_RESPONSES-',args={"values":files_list})
+                self.browser_mgr.files_list_mgr.add_to_files_list_dict('responses',file_path)
+            self.window['-FILES_LIST_RESPONSES-'].update(values=self.browser_mgr.files_list_mgr.return_display_names(key='responses'))
             response_key_list = []
-            for file_path in files_list:
+            file_paths = self.browser_mgr.files_list_mgr.get_file_path_list(key='responses')
+            for file_path in file_paths:
                 data = safe_read_from_json(file_path)
                 response_content = get_response_content(data)
                 if not isinstance(response_content,dict):
@@ -1103,7 +1097,7 @@ class GptManager:
             prompt_data = self.prompt_data_list[prompt_number]
             request_data = self.request_data_list[prompt_number]
             
-            files_list = self.window['-FILES_LIST_FILES-'].Values
+            files_list = self.browser_mgr.files_list_mgr.get_file_path_list(key='files')
             result,files_list=read_me_window(files_list)
 
             request_data+='\n\n'+result
